%--------------------------------------------------------------------------
% Anomalia gravimetrica de una esfera de radio r, contraste de densidad 
% drho, y con su centro a profundidad h
% Luego se calcula una capa equivalente dada por fuentes puntuales, los 
% cuales estan entre la superficie de medicion y el techo de la esfera.
%--------------------------------------------------------------------------
% Dampney, C.N.G., 1969. The equivalente source technique. Geophysics, 34,
% 1, 39-53.
%--------------------------------------------------------------------------

clear all
close all
clc

hold on;

xmax = -10;
xmin = -xmax;
nx   = 100;
x = linspace(xmin,xmax,nx);

for i=1:4
r = input('radio de esfera (km) [.5] ?? ');
if(isempty(r));r=.5; end;

drho = input('Contraste de densidad (gr/cc) [0.3] ?? ');
if(isempty(drho));drho=0.3; end;

h = input('Profundidad de centro de esfera (km) [1] ?? ');
if(isempty(h));h=1; end;

dg = x/h; dg = 1./ sqrt(1 + dg.* dg); dg = dg.*dg.*dg;

dg = dg * (4*pi*6.6732*r^3*drho) / (3*h^2);

%plot(x,dg,'g-');

end

figure
subplot(3,1,1)
    hold on
    plot(x,dg,'b-');
    grid on;
    xlabel('x (km)','FontSize',12);
    ylabel('dg (mgal)','FontSize',12);
    title('Anomalia gravimetrica esfera (mGal)','FontSize',12);

% Inversion por capa equivalente de la anomalia de la esfera por un conjunto
% de masas puntuales
G=6.67*1e-6;
% profundidad de la capa equivalente
H=0.3;
% estaciones gravimetricas
xobs=x;
% cantidad de monopolos
xdip=xobs;
% grilla con las estaciones y los dipolos
[XOBS,XDIP]=meshgrid(xobs,xdip);
zobs=0;
% matriz con la gravedad de los monopolos
func=G*(H-zobs)./( (XOBS-XDIP).^2+(zobs-H).^2).^(3/2);
% el problema de inversion es dg=func*m
% matriz inversa generalizada
A=inv(func'*func)*func';
% cambio de vector fila a columna
dg=dg';
% masa de la capa equivalente
m_est=A*dg;
% calculo de la gravedad de cada monopolo
gg=func*m_est;
    
%definicion de circulo
NOP=200;
xcent=0;
zcent=-h;
THETA=linspace(0,2*pi,NOP);
RHO=ones(1,NOP)*r;
[X,Y] = pol2cart(THETA,RHO);
X=X+xcent;
Y=Y+zcent;

    plot(x,gg,'r.')
    legend('Esfera','Capa equivalente de fuentes puntuales')

subplot(3,1,2)
    hold on
    plot(x,0,'k','LineWidth',10)
    plot(x,-H,'.r')
    plot(X,Y,'b');
    axis equal
    title('Esfera y capa equivalente')
    %axis([xmin xmax -2 0])
    grid on
    
subplot(3,1,3)
    plot(x,(dg-gg)*1e3)
    xlabel('Distancia')
    title('Diferencia capa equivalente - anomalia sintetica (uGal)');
    grid on